#!/usr/bin/perl

# kmailpt.pl (c) 2007 J.C. Cardot - jice<at>lea-linux.org
#
# kmailpt takes an email on stdin, performs actions based
# on this mail, then output an email on stdout.
# Without any option, it will remove any file attached.
#
# possible actions are:
# - detach attachments (save and remove from the email)
# - remove all attachments from the email
# - remove the attachment chosen by the user from the email


use Getopt::Std;
use MIME::Parser;
use MIME::WordDecoder;
use File::Copy;
use POSIX qw(strftime);

use vars qw( $PROGRAM $VERSION );
$PROGRAM   = "KMail power tools";
$VERSION   = "0.1";
$COPYLEFT  = "(c) 2007 Jean-Christophe Cardot <kmailpt@cardot.net>\nThis program is distributed under the terms of the GPL licence";

# options parsing
my $usage = "Usage: kmailpt [options]

kmailpt takes an email on stdin, performs actions based
on this mail, then output an email on stdout.
Without any option, it will remove any file attached.

Options:
        -d   detach the attachments (save and remove)
        -q   ask which attachment to remove
        -h   prints this help\n";
sub usage { print STDERR "$PROGRAM $VERSION\n$COPYLEFT\n$usage"; }
unless (getopts('dqah')) { usage(); exit 1 };

if ($opt_h) { usage(); exit 0 }

my $parser = new MIME::Parser;

# table of attachments
my @attach;

# list of removed attachments
my $removed;

# destination of saved files (option -d)
my $destdir;

my $output = $ENV{'HOME'} . '/tmp';
$parser->output_dir($output);
# Keep parsed message bodies in core (default outputs to disk):
#$parser->output_to_core(1);

# slurp the message
$/= undef;
my $msg = <>;

sub cleanup { $parser->filer->purge; }
sub exitonerror { print $msg; cleanup(); exit 0; }

sub remove_requested {
	my $att = shift;
	foreach my $a (@attach) {
		return 1 if $a eq $att;
	}
	0;
}

# keep only parts without a filename
sub keep_part {
	my $wd = default MIME::WordDecoder;
	my $part = shift;
	my $filename = $wd->decode($part->head->recommended_filename);
	if ($filename and remove_requested($filename)) {
		$removed .= ($removed?", ":"") . $filename;
		return 0;
	}
	1;
}

# and do it recursively
sub keep_parts {
	my $entity = shift;
	my @parts = $entity->parts;
	if (@parts) {
		my @keep = grep { keep_part($_) } @parts;
		$entity->parts(\@keep);
	}
	foreach my $part ($entity->parts) {
		if ( ($part->effective_type == "multipart/alternative")
		  or ($part->effective_type == "multipart/mixed")
		  or ($part->effective_type == "multipart/related")
		  or ($part->effective_type == "message/rfc822") ) {
			$part = keep_parts($part);
		}
	}
	return $entity;
}


sub get_attachment_name {
	my $wd = default MIME::WordDecoder;
	my $part = shift;
	my $filename = $wd->decode($part->head->recommended_filename);
	if ($filename) {
		#$attach .= ($attach?" ":"") . $filename . " " . $filename . " " . $filename;
		push @attach, $filename;
	}
}

sub get_attachment_names {
	my $entity = shift;
	my @parts = $entity->parts;
	if (@parts) {
		grep { get_attachment_name($_) } @parts;
	}
	foreach my $part ($entity->parts) {
		if ( ($part->effective_type == "multipart/alternative")
		  or ($part->effective_type == "multipart/mixed")
		  or ($part->effective_type == "multipart/related")
		  or ($part->effective_type == "message/rfc822") ) {
			get_attachment_names($part);
		}
	}
}

# recursively add the disclaimer
# in text/plain and text/html parts
sub add_disclaimer {
	my $entity = shift;
	foreach my $part ($entity->parts) {
		#TODO: recursively find parts to add message
		if (my $body = $part->bodyhandle) {
			# read the body text and alter it
			my $IO = $body->open("r") or exitonerror();
			my @msg = $IO->getlines;
			my $msg = join("\n", @msg);
			my $disc = `whoami`; $disc =~ s/\n//;
			$disc .= strftime(" (%d/%m/%Y %H:%M:%S)", localtime);
			my $dischtml;
			if ($opt_d) {
				my $savedtohtml;
				my $savedto;
				foreach my $a (@attach) {
					$savedto .= ($savedto?", ":"") . "file://$destdir/$a";
					$savedtohtml .= ($savedtohtml?", ":"") . "<a href='file://$destdir/$a'>$a</a>";
				}
				$dischtml = "[Attachment".(@attach>1?'s':'')." saved by $disc to $savedtohtml]";
				$disc = "[Attachment".(@attach>1?'s':'')." saved by $disc to $savedto]";
			} else {
				$dischtml = $disc = "[Attachment".(@attach>1?'s':'')." removed by $disc: $removed]";
			}
			if ("text/plain" eq $part->effective_type) {
				$msg .= "\n\n$disc\n"; 
			} elsif ("text/html" eq $part->effective_type) {
				$dischtml = "\n<p><span style='font-weight:bold'>$dischtml</span></p>\n";
				if ($msg =~ m|</body>|i) {
					$msg =~ s|</(body)>|$dischtml</$1>|i;
				} elsif ($msg =~ m|</html>|i) {
					$msg =~ s|</(html)>|$dischtml</$1>|i;
				} else {
					$msg .= "$dischtml\n";
				}
			}
			$IO->close || die "close I/O handle: $!";
			if (  ("text/plain" == $part->effective_type)
			   or ("text/html" == $part->effective_type)) {
				# write back the text to the body
				$IO = $body->open("w") or exitonerror();
				$IO->print($msg);
				$IO->close             or exitonerror();
			}
		} elsif ( ($part->effective_type == "multipart/alternative")
		  or ($part->effective_type == "multipart/mixed")
		  or ($part->effective_type == "multipart/related")
		  or ($part->effective_type == "message/rfc822") ) {
			add_disclaimer($part);
		}
	}
}


# Parse input:
$entity = $parser->parse_data($msg) or exitonerror();
#$entity = $parser->parse(\*STDIN) or die "parse failed\n";

# Take a look at the top-level entity (and any parts it has):
#$entity->dump_skeleton;

get_attachment_names($entity);

# ask the user where to save the attachments (option -d)
if ($opt_d) {
	$destdir = `kdialog --getexistingdirectory \$HOME --title "Select a directory where to save the attachments"` if $opt_d;
	$destdir =~ s/\n//;
	# exit if the user pressed cancel
	if (not $destdir) {
		print $msg;
		cleanup();
		exit 0;
	}
}

# ask the user for attachments to remove if requested by -q option
if ($opt_q or $opt_d) {
	my $cmd = 'kdialog --checklist "Select the attachments to '.($opt_q?'remove':'detach to '.$destdir).'" --title "KMail Power Tools v'.$VERSION.' (c) 2007 JCC" -- ';
	foreach my $a (@attach) { $cmd .= "\"$a\" \"$a\" \"$a\" "; }
	$ans = `$cmd`; $ans =~ s/\"[^\w\d\-_.]*\n//i; $ans =~ s/^\"//;
	# build the table of requested attachments
	@attach = split('" "', $ans);
}

if (@attach > 0) {
	# save the attachments if requested (option -d)
	if ($destdir) {
		foreach my $a (@attach) {
			copy($output.'/'.$a, $destdir.'/'.$a) or exitonerror();
		}
	}
	
	# remove the requested attachments
	$entity = keep_parts($entity);

	# if there were no attachments, then do nothing
	if ($removed) {
		# add [Removed attachments: ...] in text/plain & text/html parts
		add_disclaimer($entity);
		# output the entity
		$entity->print(\*STDOUT);
	} else {
		print $msg;
	}
} else {
	print $msg;
}

cleanup();
